#  <trigger>
#        <usage>perl password_reset_request_mail.pl forgot_password_templates</usage>
#        <version>1.0</version>
#        <description>IBM Rational Change でパスワードのリセット要求 E メールを送信するために使用するテンプレートです。
#                     テンプレートは ./wsconfig/templates/pt/forms/forgot_password_templates ディレクトリーにあります。
#        </description>
#        <argument1>
#                <name>template_folder_name</name>
#                <required>true</required>
#                <type></type>
#                <description>テンプレートを含むフォルダー名。</description>
#        </argument1>
#  </trigger>

package main;

# Includes 

## CS API Here
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;
use Text::BasicTemplate;
use CGI;
use ChangeSynergy::DefaultHandler;
use ChangeSynergy::DefaultValidation;

## This is the global variable that holds the current delivery handler module.  
my $deliverHandler="UNSET";
my $validationHandler="UNSET";

$template_dir = "./forgot_password_templates";

if(@ARGV > 1) {

      
		my $template_arg = shift;
        $template_dir = '../templates/pt/forms/' . $template_arg;
}

#Parse Trigger

# getTriggerDetails

## Parse the trigger file
my $trigger_file = shift;
$trigger = new ChangeSynergy::TriggerParser($trigger_file);

## Get the relevant data out of the Trigger Parser
my $smtp_server = $trigger->get_smtp_server() || '';
my $from_email_addr = $trigger->get_from_email_addr() || '';
my $link_expire_time = $trigger->get_link_expire_time() || '';
my $base_url = $trigger->get_base_url();
my $trig_user_details = $trigger->get_user_details();

# load the email info
$dict{'smtp_server'} = $smtp_server;
$dict{'from_email_addr'} = $from_email_addr;
$dict{'user_details'} = &arrangeData;
$dict{'expireTime'} = &arrangeExipreTime;
$dict{'to_email_addr'} = $trigger->get_to_mail_addr();
$dict{'base_url'} = $base_url;

# Load the plugin register methods
$dict{'loadfpData'} = \&loadfpData;
$dict{'registerDeliveryHandler'} = \&registerDeliveryHandler;
$dict{'registerContentPlugin'} =  \&registerContentPlugin; 
$dict{'registerValidationPlugin'} =  \&registerValidationPlugin; 

# Setup CS Connection once
$csapi = new ChangeSynergy::csapi();
$csapi->setUpConnection($base_url);
&handleTemplate($template_name);

sub registerDeliveryHandler
{
	my $plugin = shift;
    eval "use $plugin;";
	my $Handler =  $plugin . '::deliverTemplate';
	my $initPlugin = $plugin . '::initPlugin';

	if(!defined(&$Handler))
	{
		print "Delivery Handler $Handler does not define deliverTemplate...\n";
		print "Using Default Handler: $deliverHandler\n";
       	return;
    }

	if(!defined(&$initPlugin))
	{
		print "Delivery Handler $plugin ($initPlugin) does not define initPlugin...\n";
		print "Using Default Handler: $deliverHandler\n";
       	return;
    }

	## Init the plugin with the rest of the stuff from the registr.
	unless(&$initPlugin(@_))
	{
		return;
	}

	$deliverHandler = $Handler;
	return;
}

sub registerContentPlugin
{
	my $plugin = shift;
	my $bt = shift;
	print "Registering Plugin: $plugin\n";
    eval "use $plugin;";
	my $initPlugin = $plugin . '::initPlugin';
    my $Methods =  $plugin . '::Methods';

	if(!defined(&$initPlugin))
	{
		print "Content Plugin $plugin does not define initPlugin...\n";
		print "Plugin $plugin will not be available\n";
        return;
    }

	## Init the plugin with the rest of the stuff from the registr.
	unless(&$initPlugin(\@_))
	{
		print "Plugin $plugin could not be initialized!\n";
		# return;
	}

	if(!defined(&$Methods))
	{
		print "Plugin $plugin does not define Methods...\nPlugin will not be available\n";
        return;
    }

	my @methods = &$Methods;

	## Load the Methods from the plugin ...
    ## We really should do some name space management here ... like prefix the pluginame to the method somehow ...
	for $methodName (@methods)
	{
		my $completeMethod = $plugin . '::' . $methodName;
        my $longMethodName = $plugin . '__' . $methodName;

		$$bt{$methodName} = \&$completeMethod;
        $$bt{$longMethodName} = \&$completeMethod;
	}
}

sub registerValidationPlugin 
{
	my $plugin = shift;
    eval "use $plugin;";

    my $Handler =  $plugin . '::validateData';
	my $initPlugin = $plugin . '::initPlugin';

	if(!defined(&$Handler))
	{
		print "Validation Handler $Handler does not define validateData...\n";
		print "Using Default Handler: $validationHandler\n";

       	return;
    }

	if(!defined(&$initPlugin))
	{
		print "Validation Handler $plugin ($initPlugin) does not define initPlugin...\n";
		print "Using Default Handler: $validationHandler\n";

       	return;
    }

	## Init the plugin with the rest of the stuff from the registr.
	unless(&$initPlugin(@_))
	{
		return;
	}
    
	$validationHandler = $Handler;

	return;
}

$handleTemplate = 1;

sub doNotDeliver
{
  $handleTemplate = 0;
}

sub handleTemplate
{
	$tmpl = shift;
	
	## set the default Handler ... email handler.
	registerDeliveryHandler('ChangeSynergy::DefaultHandler');
    registerValidationPlugin('ChangeSynergy::DefaultValidation');

	$bt = Text::BasicTemplate->new(strip_email_comments => 1, default_undef_identifier => 1);
	# We need this so that we can at least warn about undefined values in the template
    $bt->{default_undef_identifier} = 'Undefined Attribute';
    # This makes it easy to use includes in templates ... make sure the include type is virtual
    $bt->{include_document_root} = $template_dir;
    
	# By default handle the sending of the template if there is a valid
	# handler.  If the template calls doNotDeliver(), then don't handle it.

	$handleTemplate = 1;
	my $template = $bt->parse($tmpl,\%dict, );

    if (&$validationHandler(\%dict) && $handleTemplate)
	{
		&$deliverHandler($template);
	}
	else
	{
        print "INFO: template validation rules prohibit sending expanded template to handler\n";
    }
}

sub loadfpData
{
	## If we already have the value for the attribute loaded from the trigger file, 
    ## don't retrieve it.  This makes it as fast as possible.
	    $$dict{'user_details'} = &arrangeData;
	    $$dict{'expireTime'} = &arrangeExipreTime;
	    $$dict{'base_url'} = $base_url;
}

sub arrangeData
{  
    my @userDetails = split(/\n/, $trig_user_details);
	my $outputText = "";
	my $length = @userDetails;
		
	foreach $details_per_row (@userDetails)
	{
 		my @values = split(/\|/, $details_per_row);
 		
 		if ($length > 1)
		{
	 		$outputText .= "<tr bgcolor=\"#010165\"><td colspan=\"2\"/>" . "<tr>" . "<td valign=\"top\" align=\"center\" nowrap>$values[0]</td>";
	    	
	    	if ($values[1] =~ m/http/) 
	    	{
			  $outputText .= "<td><a href=\"$values[1]\"><u>Click here to reset your password</u></a></td> </tr>";
			} 
			else 
			{
			  $outputText .= "<td>$values[1]</td></tr>";
			}
			
			$template_name = "$template_dir/password_reset_request.tmpl";
		}
		else 
		{
			$outputText .= "<b>Password Reset Request for " . $values[0] . "</b><br/><br/>";
			
			if ($values[1] =~ m/http/) 
	    	{
				$outputText .= "<div><a href=\"$values[1]\"><u>Click here to reset your password</u></a></td></div>";
	    	}
			else
			{
				$outputText .= "<div>$values[1]</div>";	
			}	
			
			$template_name = "$template_dir/password_reset_request_single_user.tmpl";
		}
	}
	
	return $outputText;
}

sub arrangeExipreTime
{
	my @userDetails = split(/\n/, $trig_user_details);
	my $length = @userDetails;
 	my $outputText = "";
 	
 	if ((length $link_expire_time) > 0)
 	{
    	$outputText = "<br/><div width=\"100%%\" colspan=\"2\">$link_expire_time</div>";
 	}
    
    return $outputText;
}
