#  <trigger>
# 	<usage>perl transitionParent_noSecurity.pl reportName stateName</usage>
# 	<version>4.4</version>
# 	<description>成功转变子 CR 之后，运行报告以获取与当前 CR 关联的父 CR。
#		然后，尝试将每个父 CR 转变为状态名称指定的状态。触发器不会检查转变是否可用。它会创建转变名称，并尝试转变父 CR。该触发器用于使用隐藏转变的 ALM 生命周期。
#   </description>
# 	<argument1>
# 		<name>reportName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>运行以查找父 CR 的报告的名称。</description>
#        </argument1>
# 	<argument2>
# 		<name>stateName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>父 CR 应转变为的最终状态。</description>
#        </argument2>
#  </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

#Get the name of the report to run and the state to transition to.
my $report = shift;
my $stateName = shift;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $
				      trigger->get_token(), $trigger->get_database());

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";
my $data = $csapi->QueryData($user, $report, $queryString, undef, undef, undef);

#Get the child data, this is the associated data, the sub-report data.
my $childData = $data->getDataObject(0)->getChildData();

#For each csapi_object_vector_position under the csapi_cobject_vector_assoc tag.
for(my $j = 0; $j < $childData->getDataSize(); $j++)
{
	#Get the object vector
	my $objVec = $childData->getDataObject($j);

	#The name of the transition is the current state of the object, the string "2", and the state to transition to
	my $transitionName = "HIDDEN_" . $objVec->getDataObjectByName("crstatus")->getValue() . "2" . $stateName;

	#Get the transition show form.
	my $tmp = $csapi->TransitionCRData($user, $objVec->getDataObjectByName("problem_number")->getValue(), $transitionName);	

	#Set the crstatus to the state we want the CR to end up in.
	$tmp->getDataObjectByName("crstatus")->setValue($stateName);

	#Do the transition
	$csapi->TransitionCR($user, $tmp);

	print "Automatically transitioned CR " . $objVec->getDataObjectByName("problem_number")->getValue() . " to $stateName.\n";
}
