//This script operates for the target_release, discovered_in_release and for baseline attributes,
//all other attributes that are sent to this script will return only the 
//listboxDefault value. Listbox values are queried from Synergy database.
//
//Pre-defined variables
//ancestors      - name and value information for parent, grand parent, etc.
//              - type: Map
//attributeName  - the name of the attribute that values are to be retrieved for.
//                type: String
//database       - the path of the database the user is currently connected to.
//                type: String
//formType       - the type of form the attribute is on. If QUERY then all results should be
//                returned.
//                type: String
//listboxDefault - the default listbox value (i.e. Any)
//                type: String
//log            - allows the script to print to the event.log
//                type: ScriptLogger (has methods logMessage(string) and logMessage(Throwable)
//results        - where the results of the script should go.
//                type: List of Strings, starts empty.
//userGroups     - a list of groups that the user is a member of.
//                type: List of Strings.
//userName       - the name of the user trying to get the listbox values.
//                type: String
//token       	- the user token
//                type: String

//Import the java.util package so we can easily use the Collections class.
importPackage(java.util);
//Import the Change util package and wslet package to call APIs to get release and baseline values.
//these packages are not for public (customer) use. 
importPackage(Packages.com.telelogic.cs.util);
importPackage(Packages.com.telelogic.cs.wslet);

//Create global variables for the objects instead of constantly looking it up from the 
//bsf object.
var results = bsf.lookupBean("results");
var database = bsf.lookupBean("database");
var userName = bsf.lookupBean("userName");
var token = bsf.lookupBean("token");
var attributeName = bsf.lookupBean("attributeName");
var listboxDefault = bsf.lookupBean("listboxDefault").toString();

//Add the default listbox value (Any by default, can be configured in the pt.cfg file) to
//the beginning of the results list. This should be called after any sorting has taken
//place if any.
function addListboxDefaultValue()
{
	//Add the listbox default value as the first element in the Array, position 0.
	//Sometimes the Bean Scripting Framework (BSF) does strange things so you have to 
	//adjust your code to account for it. Instead of just adding the listboxDefault value
	//directly to the script the .toString method had to be used so it would actually put in 
	//the correct value instead of adding null.
	results.add(0, listboxDefault);
}

//get the release values by calling Change API.
function getReleaseValues()
{
		// This call is not a public API and may change in future releases.
	return token != null ?
		PTListBox.getListboxValues(token, 'release') :
		Collections.emptyList();
}

//get the baseline values specific to a release, by calling Change API.
function getReleaseSpecificBaselineValues(releaseValue)
{
	var baselines = new ArrayList();

	if (!listboxDefault.equals(releaseValue) && (releaseValue != ''))
	{
		var cmBuildUtils = new CmBuildUtils(database, userName);
		
		try
		{
			//the code being called is not for public (customer) use. 
			var baselineList = cmBuildUtils.getBaselinesForRelease(releaseValue);
	
			for (var i = 0; i < baselineList.size(); i++)
			{
				baselines.add(baselineList.get(i).get("name"));
			}
		}
		catch(e)
		{		
		}	
	}
	
	return baselines;
}

//Combine two lists together making sure no duplicates get added.
function combineLists(master, toAdd)
{
	var size = toAdd.size();
	
	for (var i = 0; i < size; i++)
	{
		if (!master.contains(toAdd.get(i)))
		{
			master.add(toAdd.get(i));
		}
	}
}

//Main portion of the script.

var ancestorMap = new HashMap();
ancestorMap = bsf.lookupBean("ancestors");

//For the release attributes target_release, discovered_in_release,
// there will not be any ancestors. Release attributes are parent attributes
// and baseline attributes are child attributes.
if (ancestorMap.isEmpty())
{
	combineLists(results, getReleaseValues(attributeName));
}
else
{
	var keysh = ancestorMap.keySet().toArray();
		
	var parentValue = ancestorMap.get(keysh[0]);	

	//Make sure the parent value exists before trying to use it.
	if (parentValue != null)
	{
		combineLists(results, getReleaseSpecificBaselineValues(parentValue));		
	}
			
	//Sort the listbox values in ascending order using the Collections.sort
	//method. This allows the above adding of the values to be in any order.
	Collections.sort(results);
	
	//Add the default listbox value to the list created above, it is possible that
	//the default could be the only available value if the parent is not set
	//to one of the values defined above.
	addListboxDefaultValue();
}
